package dev;

import dev.util.ServerValidator;

public enum Command {
    BYE("BYE", "Sitzung beenden", null),
    LOWERCASE("LOWERCASE", "Zeichenkette in Kleinbuchstaben umwandeln", String.class),
    REVERSE("REVERSE", "Zeichenkette umkehren", String.class),
    SHUTDOWN("SHUTDOWN", "Sitzung beenden und Server herunterfahren", String.class),
    UPPERCASE("UPPERCASE", "Zeichenkette in Großbuchstaben umwandeln", String.class);

    private final String textValue;
    private final String description;
    private final Class paramType;

    String getDescription() {
        return description;
    }

    String getParamTypText() {
        return paramType == null ? null : paramType.getSimpleName();
    }

    String getTextValue() {
        return textValue;
    }

    public static final String DELIMITER = " ";

    Command(String value, String description, Class paramType) {
        this.textValue = value;
        this.description = description;
        this.paramType = paramType;
    }

    static String getCmdInfos() {
        StringBuilder builder = new StringBuilder();

        builder.append(String.format("\n%10s\t%10s\n", "Befehl", "Parameter"))
                .append("----------------------------------------------\n");

        for (Command cmd : Command.values()) {
            int maxBytes = ServerValidator.maxParamBytesInCmd(cmd);

            builder
                    .append(String.format("%10s\t <%s%s>\n",
                            cmd.getTextValue(),
                            cmd.getParamTypText(),
                            maxBytes == 0 ? "" : String.format(" (max. %d bytes)", maxBytes)
                    ))
                    .append(String.format("%10s\t %s", "", cmd.getDescription()))
                    .append("\n\n");
        }
        builder.append("----------------------------------------------\n");

        return builder.toString();
    }
}
