package dev.util;

import dev.Command;

import java.util.Arrays;
import java.util.function.Predicate;

public final class ServerValidator {

    static final int PORT_MIN = 0;
    static final int PORT_MAX = 65535;
    public static final int BYTES_MAX = 255;
    private static final int[] PORTS_RESERVED = new int[]{0};

    public static boolean isPortValid(String portString) {
        try {
            int port = Integer.parseInt(portString);
            return portValidator.test(port);
        } catch (NumberFormatException e) {
            return false;
        }
    }

    private static final Predicate<Integer> portValidator = port -> {
        if (port < PORT_MIN) return false;
        if (port > PORT_MAX) return false;
        return Arrays.stream(PORTS_RESERVED).noneMatch(i -> i == port);
    };

    public static boolean isPwValid(String pw) {
        return pwValidator.test(pw);
    }

    private static final Predicate<String> pwValidator = pw -> {
        if (pw.isEmpty()) return false;
        if (pw.contains("\n")) return false;
        return StringHelper.countBytesInString(pw) <= maxParamBytesInCmd(Command.SHUTDOWN);
    };

    public static boolean PARAM_VALID(Command cmd, String parameter) {
        if (parameter == null || parameter.isEmpty()) return false;
        if (cmd.equals(Command.BYE)) return false;

        return StringHelper.countBytesInString(parameter) <= maxParamBytesInCmd(cmd);
    }

    public static int maxParamBytesInCmd(Command cmd) {
        if (cmd.equals(Command.BYE)) return 0;
        return BYTES_MAX - StringHelper.countBytesInString(cmd.name()) - StringHelper.countBytesInString(Command.DELIMITER);
    }

}
