package dev.util;

import org.junit.jupiter.api.Test;

import static dev.util.ServerExceptions.SyntaxError;
import static dev.util.StringHelper.*;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

class StringHelperTest {

    @Test
    void lowercaseFromLowercase() throws Exception {
        assertEquals("abc", toLowerCase("abc"));
    }

    @Test
    void lowercaseNoEmptyString() {
        assertThrows(SyntaxError.class, () -> toLowerCase(""));
    }

    @Test
    void lowercaseNoNullString() {
        assertThrows(SyntaxError.class, () -> toLowerCase(null));
    }

    @Test
    void lowercaseFrom1ASCII() throws Exception {
        assertEquals("z", toLowerCase("Z"));
    }

    @Test
    void lowercaseFromAlphabet() throws Exception {
        assertEquals("abcdefghijklmnopqrstuvwxyz", toLowerCase("ABCDEFGHIJKLMNOPQRSTUVWXYZ"));
    }

    @Test
    void lowercaseFromSpecialChar() throws Exception {
        assertEquals("!", toLowerCase("!"));
    }

    @Test
    void lowercaseFromMixedCase() throws Exception {
        assertEquals("abcdefg", toLowerCase("AbCdEfG"));
    }

    @Test
    void lowercaseFromMixedChars() throws Exception {
        assertEquals("a!b?c§d&e", toLowerCase("A!B?C§D&E"));
    }

    @Test
    void lowercaseFromSingleUmlaut() throws Exception {
        assertEquals("ä", toLowerCase("Ä"));
    }

    @Test
    void lowercaseFromUmlauts() throws Exception {
        assertEquals("äöü", toLowerCase("ÄÖÜ"));
    }

    @Test
    void lowercaseFromLigature() throws Exception {
        assertEquals("ß", toLowerCase("ß"));
    }


    @Test
    void uppercaseFromUppercase() throws Exception {
        assertEquals("ABC", toUpperCase("ABC"));
    }

    @Test
    void uppercaseNoEmptyString() {
        assertThrows(SyntaxError.class, () -> toUpperCase(""));
    }

    @Test
    void uppercaseNoNullString() {
        assertThrows(SyntaxError.class, () -> toUpperCase(null));
    }

    @Test
    void uppercaseFromSingleASCII() throws Exception {
        assertEquals("Z", toUpperCase("z"));
    }

    @Test
    void uppercaseFromASCIIAlphabet() throws Exception {
        assertEquals("ABCDEFGHIJKLMNOPQRSTUVWXYZ", toUpperCase("abcdefghijklmnopqrstuvwxyz"));
    }

    @Test
    void uppercaseFromSpecialChar() throws Exception {
        assertEquals("!", toUpperCase("!"));
    }

    @Test
    void uppercaseFromMixedCase() throws Exception {
        assertEquals("ABCDEFG", toUpperCase("AbCdEfG"));
    }

    @Test
    void uppercaseFromMixedChars() throws Exception {
        assertEquals("A!B?C§D&E", toUpperCase("a!b?c§d&e"));
    }

    @Test
    void uppercaseFromSingleUmlaut() throws Exception {
        assertEquals("Ä", toUpperCase("ä"));
    }

    @Test
    void uppercaseFromUmlauts() throws Exception {
        assertEquals("ÄÖÜ", toUpperCase("äöü"));
    }

    @Test
    void uppercaseFromLigature() throws Exception {
        assertEquals("GROSS", toUpperCase("groß"));
    }


    @Test
    void reverseSingleChar() throws Exception {
        assertEquals("a", reverseString("a"));
    }

    @Test
    void reverseNoEmptyString() {
        assertThrows(SyntaxError.class, () -> reverseString(""));
    }

    @Test
    void reverseNoNullString() {
        assertThrows(SyntaxError.class, () -> reverseString(null));
    }

    @Test
    void reverseEvenNumberOfLetters() throws Exception {
        assertEquals("dcba", reverseString("abcd"));
    }

    @Test
    void reverseUnevenNumberOfLetters() throws Exception {
        assertEquals("edcba", reverseString("abcde"));
    }

    @Test
    void reverseMixedChars() throws Exception {
        assertEquals("E&D§C?B!A", reverseString("A!B?C§D&E"));
    }

    @Test
    void countBytesInStringMaxLength() {
        assertEquals(255, countBytesInString(generateSingleByteString('x', 255)));
    }

    @Test
    void countBytesInEmptyString() {
        assertEquals(0, countBytesInString(""));
    }

    @Test
    void countBytesInNullString() {
        assertEquals(0, countBytesInString(null));
    }

    @Test
    void countBytesInControlChars() {
        assertEquals(3, countBytesInString("\n\n\n"));
    }

    @Test
    void count2ByteChars() {
        assertEquals(2 * 3, countBytesInString("äöü"));
    }

    @Test
    void count3ByteChars() {
        assertEquals(3 * 3, countBytesInString("☺☻♥"));
    }

    @Test
    void count4ByteChars() {
        assertEquals(4 * 3, countBytesInString("𩸽𩸽𩸽"));
    }

    @Test
    void countBytesMixedSizeChars() {
        assertEquals(1 + 2 + 3 + 4, countBytesInString("aä☺𩸽"));
    }

}